#!/usr/bin/env node

import { dirname, join } from 'path';
import fs from 'fs-extra';
import chalk from 'chalk';
import glob from 'fast-glob';
import { fileURLToPath } from 'url';

console.log("Building a new Bulldog and Fisk");

const cwd = process.cwd();

// The base of this repo -- we're copying files from there.
//
const bBase = join(dirname(fileURLToPath(import.meta.url)), '../');

console.log("bb", bBase, cwd);

const appMap = new Map();
const gConfig = {
    dot: false,
    onlyFiles: true,
    absolute: true,
    ignore: ['**/node_modules']
};

// Set /actors paths.
//
await add(join(bBase, 'actors/**/*.js'));
await add(join(bBase, 'openApi/**/*.yml'));
await add(join(bBase, '.editorconfig'));
await add(join(bBase, '.gitignore'));
await add(join(bBase, '.yarnrc'));
await add(join(bBase, 'LICENSE.md'));
await add(join(bBase, 'package.json'), content => {
    content.author = {
        "name": "",
        "email": "",
        "url": ""
    };
    content.dependencies = {};
    content.name = "";

    return content;
});

for(const [ targetStub, content ] of appMap.entries()) {
    const targetPath = join(cwd, targetStub);
    if(await fs.pathExists(targetPath)) {
        console.log(chalk.bgRed.white.bold(`\n Error: FILE ALREADY EXISTS \n Can't generate source file [${targetStub}] \n into ${targetPath} \n The idea is to run '> bf create' within an empty directory, once. `));
    } else {
        //await fs.write(targetPath, content);
        console.log(chalk.bgGreenBright.black.bold(`\n Generated source file [${targetStub}] \n into ${targetPath} \n`));
    }
}

async function add(pattern, formatter=f=>f) {
    for(const actorFile of await glob(pattern, gConfig)) {
        const relativePath = actorFile.replace(bBase, '');
        let content = await fs.readFile(actorFile, { encoding: 'utf8' });
        try {
            content = JSON.parse(content);
        } catch (err) {}
        appMap.set(relativePath, formatter(content));
    }
}
